@@ -16,7 +16,7 @@ def test_register(client, app):
     # test that the user was inserted into the database
     with app.app_context():
         assert (
-            get_db().execute("select * from user where username = 'a'").fetchone()
+            get_db().execute("SELECT * FROM user WHERE username = 'a'").fetchone()
             is not None
         )
 
